// TvvO

#pragma comment(lib, "dxguid.lib")
#pragma comment(lib, "d3d9.lib")
#pragma comment(lib, "d3dx9.lib")

#include <windows.h>
#include <tchar.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <vector>

#include "sprite.h"

TCHAR gName[100] = _T("2D`TvvO");
#define FRAND ((float)rand()/RAND_MAX)

class Task {
	OX::Sprite sprite;
	float degUnit;
	float scaleUnit;
	float life;
	bool useAlpha;

	class State { public: virtual bool update(Task* task) = 0; };

	class FadeIn : public State {
		bool bUpdated;
		float target_A, cur_A;
	public:
		FadeIn() : bUpdated(false), target_A(), cur_A() {}
		bool update(Task* task) {
			if (bUpdated == false) {
				target_A = task->sprite.getAlpha();
				task->sprite.setAlpha(0.0f);
				bUpdated = true;
			}
			cur_A += 0.01f;
			if (cur_A >= target_A)
				cur_A = target_A;
			task->sprite.setAlpha( cur_A );
			if (cur_A >= target_A) {
				delete this;
				task->state = new Idle;
				return false;
			}
			return false;
		}
	};

	class Idle : public State {
	public:
		bool update(Task* task) {
			task->life -= 1.0f;
			if (task->life <= 0.0f) {
				delete this;
				if (task->useAlpha)
					task->state = new FadeOut;
				else
					return true;
			}
			return false;
		}
	};

	class FadeOut : public State {
	public:
		bool update(Task* task) {
			float a = task->sprite.getAlpha();
			a -= 0.01f;
			if ( a <= 0.0f ) {
				delete this;
				return true;
			}
			task->sprite.setAlpha(a);
			return false;
		}
	};

	State *state;

public:
	Task( int scW, int scH, IDirect3DTexture9 *tex, bool useAlpha ) {
		this->useAlpha = useAlpha;
		sprite.setScreenSize( scW, scH );
		sprite.setTexture( tex, false );
		sprite.setPos( scW * FRAND, scH * FRAND );
		sprite.setPivot( 64.0f, 64.0f );
		sprite.setPriority( FRAND );
		float uvL = 0.15f + 0.85f * FRAND;
		sprite.setUV(0.50f - uvL * 0.5f, 0.50f - uvL * 0.5f, uvL, uvL);
		sprite.setSize( (int)(256 * uvL), (int)(256 * uvL) );
		sprite.setAlpha( useAlpha ? FRAND : 1.0f );

		degUnit = 1.0f * (0.5f - FRAND);
		scaleUnit = 0.02f * FRAND;
		life = 60.0f + 180.0f * FRAND;

		if (useAlpha)
			state = new FadeIn;
		else
			state = new Idle;
	}

	bool update() {
		float sx, sy;
		sprite.getScale( &sx, &sy );
		sprite.setScale( sx + scaleUnit, sy + scaleUnit );
		sprite.setRotate( sprite.getRotate() + degUnit );
		sprite.draw();
		return state->update(this);
	}
};

LRESULT CALLBACK WndProc(HWND hWnd, UINT mes, WPARAM wParam, LPARAM lParam){
   if(mes == WM_DESTROY) {PostQuitMessage(0); return 0;}
   return DefWindowProc(hWnd, mes, wParam, lParam);
}

int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
   // AvP[V̏
   MSG msg; HWND hWnd;
   WNDCLASSEX wcex ={sizeof(WNDCLASSEX), CS_HREDRAW | CS_VREDRAW, WndProc, 0, 0, hInstance, NULL, NULL,
                                    (HBRUSH)(COLOR_WINDOW+1), NULL, (TCHAR*)gName, NULL};
   if(!RegisterClassEx(&wcex))
      return 0;

   int w = 800;
   int h = 600;
   RECT r = {0, 0, w, h};
   ::AdjustWindowRect( &r, WS_OVERLAPPEDWINDOW, FALSE );
   float cw = (float)(r.right - r.left);
   float ch = (float)(r.bottom - r.top);
   if(!(hWnd = CreateWindow(gName, gName, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, (int)cw, (int)ch, NULL, NULL, hInstance, NULL)))
      return 0;

   // Direct3D̏
   LPDIRECT3D9 g_pD3D;
   LPDIRECT3DDEVICE9 dev;
   if( !(g_pD3D = Direct3DCreate9( D3D_SDK_VERSION )) ) return 0;

   D3DPRESENT_PARAMETERS d3dpp = {w, h, D3DFMT_UNKNOWN, 0, D3DMULTISAMPLE_NONE, 0, D3DSWAPEFFECT_DISCARD, NULL, TRUE, 0, D3DFMT_UNKNOWN, 0, 0}; 

   if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &dev ) ) ) {
      g_pD3D->Release();
      return 0;
   }

   // XvCg
   OX::Sprite::begin_first( dev );

   // eNX`
   IDirect3DTexture9 *tex = 0;
   D3DXCreateTextureFromFile( dev, "resource/test.png", &tex);
   IDirect3DTexture9 *bgTex = 0;
   D3DXCreateTextureFromFile( dev, "resource/ox_bg.jpg", &bgTex);

   // wiXvCg
   OX::Sprite bgSprite( w, h );
   bgSprite.setSize( w, h );
   bgSprite.setUV(0.0f, 0.0f, 6.0f, 6.0f); // JԂɂȂĂ̂I
   bgSprite.setTexture( bgTex, false );
   bgSprite.setPriority( 1.0f );

   // ^XNXg
   std::list<Task*> tasks;
   std::list<Task*> delTasks;	// 폜p

   ShowWindow(hWnd, nCmdShow);

	// bZ[W [v
   bool useAlpha = false;	// trueɂƃtF[hCAEg܂
   float elapsed = 0.0f;
	do{
		if( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) ){
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		// Direct3D̏
		dev->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(70,122,36), 1.0f, 0 );
		dev->BeginScene();

		// wi`
		bgSprite.draw();

		// ^XN
		elapsed += 1.0f;
		if (elapsed >= 10.0f) {
			tasks.push_back(new Task((int)w, (int)h, tex, useAlpha));
			elapsed = 0.0f;
		}

		// ^XN
		std::list<Task*>::iterator it = tasks.begin();
		for ( ; it != tasks.end(); ) {
			if ((*it)->update() == true) {
				delTasks.push_back(*it);
				it = tasks.erase(it);
				continue;
			}
			it++;
		}

		// XvCg܂Ƃ߂ĕ`
		OX::Sprite::drawAll( dev );
		OX::Sprite::clearDrawList();

		// I^XN폜
		for (it = delTasks.begin(); it != delTasks.end(); it++)
			delete (*it);
		delTasks.clear();

		dev->EndScene();
		dev->Present( NULL, NULL, NULL, NULL );
	} while(msg.message != WM_QUIT);

	for ( std::list<Task*>::iterator it = tasks.begin(); it != tasks.end(); it++ )
		delete *it;

	OX::Sprite::end_last();

	tex->Release();
	bgTex->Release();
	dev->Release();
	g_pD3D->Release();

   return 0;
}
