#include "sprite.h"

namespace OX {
	IDirect3DVertexBuffer9 *Sprite::buf = 0;
	std::list<Sprite*> Sprite::drawObjectList;
	IDirect3DVertexDeclaration9 *Sprite::decl;
	ID3DXEffect *Sprite::effect;

	Sprite::Sprite( int screenWidth, int screenHeight ) :
		tex(),
		polyW(128), polyH(128),
		pivotX(), pivotY(),
		posX(), posY(), posZ(),
		rad(),
		scaleX(1.0f), scaleY(1.0f),
		uvLeft(), uvTop(),
		uvW(1.0f), uvH(1.0f),
		alpha(1.0f),
		bActivity(true)
	{
		setScreenSize( screenWidth, screenHeight );
	}

	Sprite::Sprite() :
		tex(),
		polyW(128), polyH(128),
		pivotX(), pivotY(),
		posX(), posY(), posZ(),
		rad(),
		scaleX(1.0f), scaleY(1.0f),
		uvLeft(), uvTop(),
		uvW(1.0f), uvH(1.0f),
		alpha(1.0f),
		bActivity(true)
	{
		setScreenSize( 640, 480 );
	}

	Sprite::Sprite( const Sprite &r ) {
		copy( r );
	}

	Sprite::~Sprite() {
		if (tex)
			tex->Release();
	}

	// Rs[
	void Sprite::copy( const Sprite &r ) {
		r.tex->AddRef();
		tex = r.tex;
		polyW = r.polyW;
		polyH = r.polyH;
		pivotX = r.pivotX;
		pivotY = r.pivotY;
		posX = r.posX;
		posY = r.posY;
		posZ = r.posZ;
		rad = r.rad;
		scaleX = r.scaleX;
		scaleY = r.scaleY;
		uvLeft = r.uvLeft;
		uvTop = r.uvTop;
		uvW = r.uvW;
		uvH = r.uvH;
		alpha = r.alpha;
		scW = r.scW;
		scH = r.scH;
		bActivity = r.bActivity;
	}

	// ʒ_obt@쐬
	void Sprite::begin_first( IDirect3DDevice9* dev ) {
		if (buf == 0) {
			float commonVtx[] = {
				0.0f, 0.0f, 0.0f,   0.0f, 0.0f,  // 0
				1.0f, 0.0f, 0.0f,   1.0f, 0.0f,  // 1
				0.0f, 1.0f, 0.0f,   0.0f, 1.0f,  // 2
				1.0f, 1.0f, 0.0f,   1.0f, 1.0f,  // 3
			};
			dev->CreateVertexBuffer( sizeof(commonVtx), 0, 0, D3DPOOL_MANAGED, &buf, 0 );
			float *p = 0;
			if (buf) {
				buf->Lock( 0, 0, (void**)&p, 0 );
				memcpy( p, commonVtx, sizeof(commonVtx) );
				buf->Unlock();
			}
		}

		// VF[_쐬
		if (effect == 0) {
			ID3DXBuffer *error = 0;
			if ( FAILED( D3DXCreateEffectFromFile( dev, "sprite.fx", 0, 0, 0, 0, &effect, &error) ) ) {
				OutputDebugStringA( (const char*)error->GetBufferPointer());
				return;
			}
		}

		// _錾쐬
		if (decl == 0) {
			D3DVERTEXELEMENT9 elems[] = {
				{0, 0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
				{0, sizeof(float) * 3, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0},
				D3DDECL_END()
			};
			dev->CreateVertexDeclaration( elems, &decl );
		}
	}

	// Еt
	void Sprite::end_last() {
		if (buf)
			buf->Release();
		if (effect)
			effect->Release();
		if (decl)
			decl->Release();
	}

	// |TCYw
	void Sprite::setSize( int w, int h ) {
		polyW = w;
		polyH = h;
	}

	// XN[TCYw
	void Sprite::setScreenSize( int w, int h ) {
		scW = w;
		scH = h;
	}

	// s{bgw
	void Sprite::setPivot( float x, float y ) {
		pivotX = x;
		pivotY = y;
	}

	void Sprite::getPivot( float *x, float *y ) {
		*x = pivotX;
		*y = pivotY;
	}

	// pw
	void Sprite::setPos( float x, float y ) {
		posX = x;
		posY = y;
	}

	void Sprite::getPos( float *x, float *y ) {
		*x = posX;
		*y = posY;
	}

	void Sprite::setRotate( float deg ) {
		// WAɕϊ
		rad = deg / 180.0f * 3.1415926535f;
	}

	float Sprite::getRotate() {
		// fO[ŕԂ
		return rad / 3.14159265f * 180.0f;
	}

	void Sprite::setScale( float sx, float sy ) {
		scaleX = sx;
		scaleY = sy;
	}

	void Sprite::getScale( float *sx, float *sy ) {
		*sx = scaleX;
		*sy = scaleY;
	}

	// eNX`ݒ
	void Sprite::setTexture( IDirect3DTexture9 *tex, bool isResize ) {
		if (tex)
			tex->AddRef();
		if (this->tex)
			this->tex->Release();
		this->tex = tex;

		if (isResize == true) {
			D3DSURFACE_DESC desc;
			tex->GetLevelDesc( 0, &desc );
			setSize( desc.Width, desc.Height );
		}
	}

	// UV؂w
	void Sprite::setUV( float l, float t, float w, float h ) {
		uvLeft = l;
		uvTop = t;
		uvW = w;
		uvH = h;
	}

	void Sprite::getUV_LT( float *l, float *t ) {
		*l = uvLeft;
		*t = uvTop;
	}

	void Sprite::getUV_WH( float *w, float *h ) {
		*w = uvW;
		*h = uvH;
	}

	// ݒ
	void Sprite::setAlpha( float a ) {
		alpha = a;
	}

	float Sprite::getAlpha() {
		return alpha;
	}

	// ANeBreBݒ
	void Sprite::setActivity( bool isActive ) {
		bActivity = isActive;
	}

	bool Sprite::getActivity() {
		return bActivity;
	}

	// vCIeBݒ
	void Sprite::setPriority( float z ) {
		posZ = z;
	}

	float Sprite::getPriority() {
		return posZ;
	}

	// `惊Xgɐς
	void Sprite::draw() {
		drawObjectList.push_back( this );
	}

	// `惊XgCɕ`
	void Sprite::drawAll( IDirect3DDevice9 *dev ) {
		if (buf == 0 || effect == 0 || decl == 0)
			return;	// `s

		// _obt@E_錾ݒ
		dev->SetStreamSource( 0, buf, 0, sizeof(float) * 5 );
		dev->SetVertexDeclaration( decl );

		// 2D`pˉeϊs
		D3DXMATRIX proj;
		D3DXMatrixIdentity( &proj );
		proj._41 = -1.0f;
		proj._42 =  1.0f;

		// VF[_Jn
		UINT numPass = 0;
		effect->SetTechnique( "Tech" );
		effect->Begin(&numPass, 0);
		effect->BeginPass(0);

		// `惊Xgɓo^ĂXvCgCɕ`悷
		std::list<Sprite*>::iterator it = drawObjectList.begin();
		for (; it != drawObjectList.end(); it++) {
			Sprite* sp = (*it);
			if (sp->bActivity == false)
				continue;

			proj._11 =  2.0f / sp->scW;
			proj._22 = -2.0f / sp->scH;

			D3DXMATRIX world, scale, rot;
			D3DXMatrixScaling( &world, (float)sp->polyW, (float)sp->polyH, 1.0f );	// |STCY
			D3DXMatrixScaling( &scale, sp->scaleX, sp->scaleY, 1.0f );	// [JXP[
			D3DXMatrixRotationZ( &rot, sp->rad );						// ]
			world._41 = -sp->pivotX;		// s{bgItZbg
			world._42 = -sp->pivotY;
			world = world * scale * rot;
			world._41 += sp->posX + sp->pivotX;	// s{bgItZbg
			world._42 += sp->posY + sp->pivotY;

			effect->SetMatrix( "world", &world );
			effect->SetMatrix( "proj", &proj );
			effect->SetTexture( "tex", sp->tex );
			effect->SetFloat( "uv_left"  , sp->uvLeft );
			effect->SetFloat( "uv_top"   , sp->uvTop );
			effect->SetFloat( "uv_width" , sp->uvW );
			effect->SetFloat( "uv_height", sp->uvH );
			effect->SetFloat( "alpha"    , sp->alpha );
			effect->CommitChanges();
			dev->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );
		}

		effect->EndPass();
		effect->End();
	}

	// `惊XgNA
	void Sprite::clearDrawList() {
		drawObjectList.clear();
	}
}